/*
 * Firedoclipse
 * 
 * Copyright (c) 2007-2008 Massachusetts Institute of Technology
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package firedoclipse;

import java.util.Map;

import org.eclipse.jface.text.*;
import org.eclipse.jface.text.hyperlink.IHyperlink;

public class AntHyperlinkDetector extends DocHyperlinkDetector {
    
    static final Map<String, String> KEYWORDS = loadKeywords("Ant");
    /**
     * URL for core task documentation.
     */
    static final String CORE_TASK_URL = KEYWORDS.get("CORE_TASK_URL");
    
    public IHyperlink[] detectHyperlinks(ITextViewer textViewer, IRegion region, boolean canShowMultipleHyperlinks) {
        if (region == null) return null;
        
        IRegion tagRegion = getSgmlTagRegion(textViewer.getDocument(), region);
        if (tagRegion == null) return null;
        
        String word;
        try {
            word = textViewer.getDocument().get(tagRegion.getOffset(), tagRegion.getLength());
        } catch (BadLocationException ble) { return null; }
        
        String url = KEYWORDS.get(word.toLowerCase());
        if (url == null) {
            return null;
        } else if (url.equals("")) {
            url = CORE_TASK_URL.replace("----", word.toLowerCase());
        }
        
        return new IHyperlink[] { new DocHyperlink(tagRegion, "http://ant.apache.org/manual/", "mainFrame", url) };
    }
}
