/*
 * Firedoclipse
 * 
 * Copyright (c) 2007-2008 Massachusetts Institute of Technology
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package firedoclipse;

import java.rmi.ConnectException;

import jsjr.ConnectorImpl;
import jsjr.Marshalable;

import org.eclipse.jface.text.IRegion;
import org.eclipse.jface.text.hyperlink.IHyperlink;

/**
 * Documentation hyperlink to be opened in a connected Firefox browser.
 * 
 * @author maxg
 */
public class DocHyperlink implements IHyperlink {
    
    private final IRegion region;
    private final String method;
    private final Marshalable[] arguments;
    
    public DocHyperlink(IRegion region, String url) {
        this.region = region;
        this.method = "browse";
        this.arguments = ConnectorImpl.marshalables(url);
    }
    
    public DocHyperlink(IRegion region, String index, String frame, String url) {
        this.region = region;
        this.method = "browseFramed";
        this.arguments = ConnectorImpl.marshalables(index, frame, url);
    }

    public IRegion getHyperlinkRegion() {
        return region;
    }

    public String getHyperlinkText() {
        return "Open Doc";
    }

    public String getTypeLabel() {
        return "Doc";
    }

    public void open() {
        try {
            Activator.getDefault().connection().call("@csail.mit.edu/firedoclipse/Browse;1", method, arguments);
        } catch (ConnectException ce) { }
    }
}
