/*
 * Firedoclipse
 * 
 * Copyright (c) 2007-2008 Massachusetts Institute of Technology
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package firedoclipse;

import java.util.Map;

import org.eclipse.jface.text.*;
import org.eclipse.jface.text.hyperlink.IHyperlink;

public class HtmlHyperlinkDetector extends DocHyperlinkDetector {
    
    static final Map<String, String> HTML_KEYWORDS = loadKeywords("HTML");
    /**
     * URL for HTML documentation.
     */
    static final String HTML_URL = HTML_KEYWORDS.get("HTML_URL");
    
    public IHyperlink[] detectHyperlinks(ITextViewer textViewer, IRegion region, boolean canShowMultipleHyperlinks) {
        if (region == null) return null;
        
        IRegion tagRegion = getSgmlTagRegion(textViewer.getDocument(), region);
        if (tagRegion == null) return null;
        
        String word;
        try {
            word = textViewer.getDocument().get(tagRegion.getOffset(), tagRegion.getLength());
        } catch (BadLocationException ble) { return null; }
        
        String url;
        if (HTML_KEYWORDS.containsKey(word.toLowerCase())) {
            url = HTML_URL.replace("_cat_", HTML_KEYWORDS.get(word)).replace("_tag_", word.toUpperCase());
        } else {
            return null;
        }
        
        return new IHyperlink[] { new DocHyperlink(tagRegion, url) };
    }
}
