/*
 * Firedoclipse
 * 
 * Copyright (c) 2007-2008 Massachusetts Institute of Technology
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package firedoclipse;

import java.net.URL;

import org.eclipse.jdt.core.*;
import org.eclipse.jdt.internal.ui.javaeditor.EditorUtility;
import org.eclipse.jdt.ui.JavaUI;
import org.eclipse.jface.text.IRegion;
import org.eclipse.jface.text.ITextViewer;
import org.eclipse.jface.text.hyperlink.IHyperlink;
import org.eclipse.ui.texteditor.ITextEditor;

/**
 * Documentation hyperlink detector for Java source files.
 * 
 * @author maxg
 */
@SuppressWarnings("restriction")
public class JavaHyperlinkDetector extends DocHyperlinkDetector {
    
    public IHyperlink[] detectHyperlinks(ITextViewer textViewer, IRegion region, boolean canShowMultipleHyperlinks) {
        if (region == null) return null;
        
        ITextEditor textEditor = (ITextEditor)getAdapter(ITextEditor.class);
        IJavaElement input = EditorUtility.getEditorInputJavaElement(textEditor, false);
        if (input == null) return null;
        
        IRegion wordRegion = getJavaWordRegion(textViewer.getDocument(), region);
        if (wordRegion == null) return null;
        
        IJavaElement element = getJavaElement((ICodeAssist)input, wordRegion);
        if (element == null) return null;
        
        URL url, index;
        try {
            url = JavaUI.getJavadocLocation(element, true);
            index = JavaUI.getJavadocBaseLocation(element);
        } catch (JavaModelException e) { return null; }
        if (url == null) return null;
        
        if (index == null) {
            return new IHyperlink[] { new DocHyperlink(wordRegion, url.toString()) };
        } else {
            return new IHyperlink[] { new DocHyperlink(wordRegion, index.toString(), "classFrame", url.toString()) };
        }
    }
    
    /**
     * Get the Java element corresponding to a region of text
     * 
     * @param input containing Java element
     * @param wordRegion region
     * @return Java element in the region, or <code>null</code> if none
     */
    protected IJavaElement getJavaElement(ICodeAssist input, IRegion wordRegion) {
        IJavaElement[] elements = null;
        try {
            elements = input.codeSelect(wordRegion.getOffset(), wordRegion.getLength());
        } catch (JavaModelException e) { return null; }
        if (elements.length != 1) return null;
        return elements[0];
    }
}
