/*
 * Firedoclipse
 * 
 * Copyright (c) 2007-2008 Massachusetts Institute of Technology
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package firedoclipse;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.Map;

import org.eclipse.jface.text.*;
import org.eclipse.jface.text.hyperlink.IHyperlink;

/**
 * Documentation hyperlink detector for JavaScript source files.
 * 
 * @author maxg
 */
public class JavaScriptHyperlinkDetector extends DocHyperlinkDetector {
    
    static final Map<String, String> KEYWORDS = loadKeywords("JavaScript");
    /**
     * URL for XPCOM interface documentation.
     */
    static final String INTERFACE_URL = KEYWORDS.get("INTERFACE_URL");
    /**
     * URL for XPCOM component documentation.
     */
    static final String COMPONENT_URL = KEYWORDS.get("COMPONENT_URL");
    
    public IHyperlink[] detectHyperlinks(ITextViewer textViewer, IRegion region, boolean canShowMultipleHyperlinks) {
        if (region == null) return null;
        
        IRegion wordRegion = getStringLiteralRegion(textViewer.getDocument(), region);
        if (wordRegion == null) wordRegion = getJavaWordRegion(textViewer.getDocument(), region);
        if (wordRegion == null) return null;
        
        String word;
        try {
            word = textViewer.getDocument().get(wordRegion.getOffset(), wordRegion.getLength());
        } catch (BadLocationException e) { return null; }
                
        String url = KEYWORDS.get(word);
        if (url != null) {
            // use the URL
        } else if (word.matches("^(ns|moz|fuel|rdf)I.*")) {
            url = INTERFACE_URL.replace("----", word);
        } else if (word.matches("^@(mozilla).*")) {
            // XUL planet:  @foo.org/bar/x-BAZquux2;1?fred=blah  ->  barxBAZquux21fredblah
            // url = COMPONENT_URL.replace("----", word.replaceFirst("@[^/]*/", "").replaceAll("\\p{Punct}", ""));
            try {
                url = COMPONENT_URL.replace("----", URLEncoder.encode(word, "UTF-8"));
            } catch (UnsupportedEncodingException uee) { return null; }
        } else {
            return null;
        }
                
        return new IHyperlink[] { new DocHyperlink(wordRegion, url) };
    }
}
