/*
 * Firedoclipse
 * 
 * Copyright (c) 2007-2008 Massachusetts Institute of Technology
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");

const Cc = Components.classes;
const Ci = Components.interfaces;
const log = function(){};

const MARKER = "firedoclipse-browse-tab";

function Browse() {
    log("Browse()\n");
    this.wrappedJSObject = this;
}

Browse.prototype = {
    classDescription: "Firedoclipse browse",
    classID:          Components.ID("{98ecc6c0-c94a-11dd-ad8b-0800200c9a66}"),
    contractID:       "@csail.mit.edu/firedoclipse/Browse;1",
    QueryInterface:   XPCOMUtils.generateQI(),
    
    windowMediator: function() {
        return Cc["@mozilla.org/appshell/window-mediator;1"].getService(Ci.nsIWindowMediator);
    },
    
    showUpdateMessage: false,
    firefoxVersion: function() {
        return Cc["@mozilla.org/fuel/application;1"].getService(Ci.fuelIApplication).extensions.get("firedoclipse@csail.mit.edu").version;
    },
    eclipseVersion: 0,
    
    eclipseVersionIs: function(version) {
        this.eclipseVersion = version;
        this.showUpdateMessage = version < this.firefoxVersion();
    },
    
    browseFramed: function(index, frame, url) {
        log("Browse.browseFramed(" + index + ", " + frame + ", " + url + ")\n");
        
        var browser = this.browse(index);
        if (browser == null) return;
        
        function go(window) {
            if (window.name == frame) {
                if (window.location.href != url)
                    window.location.href = url;
                return true;
            }
            for (var ii = 0; ii < window.frames.length; ii++) {
                if (go(window.frames[ii]))
                    return true;
            }
            return false;
        };
        
        var listener = {
            onStateChange: function(webProgress, request, stateFlags, status) {
                browser.webProgress.removeProgressListener(filter);
                go(browser.contentWindow);
            }
        };
        
        var filter = Cc["@mozilla.org/appshell/component/browser-status-filter;1"].createInstance(Ci.nsIWebProgress);
        filter.addProgressListener(listener, null);
        
        browser.webProgress.addProgressListener(filter, Ci.nsIWebProgress.NOTIFY_STATE_WINDOW);
        if ( ! browser.webProgress.isLoadingDocument) {
            listener.onStateChange(null, null, null, null);
        }
    },
    
    browse: function(url) {
        log("Browse.browse(" + url + ")\n");
        
        /*
         * XXX is there a better way to fix this?
         * Eclipse sends URLs like jar:file///docs/some.jar!/ref/api, where
         * `api` is a directory. Firefox does not load this URL, so we must
         * add a trailing `/`. But Firefox further does not look for index
         * documents using file protocols, so we add `index.html` as well.
         */ 
        if (url.match(/^jar:.*\/[^\.]+$/)) {
            url += "/index.html";
        }
        
        if ((browser = this.browseInTab(url)) != null) {
            return browser;
        }
        
        return this.browseNewTab(url);
        
    },
    
    browseInTab: function(url) {
        log("Browse.browseInTab(" + url + ")\n");
        
        var browsers = this.windowMediator().getEnumerator("navigator:browser");
        while (browsers.hasMoreElements()) {
            var tabs = browsers.getNext().getBrowser();
            for (var index = 0; index < tabs.tabContainer.childNodes.length; index++) {
                var tab = tabs.tabContainer.childNodes[index];
                if (tab.id == MARKER) { // XXX can we use getElementById?
                    var browser = tabs.getBrowserForTab(tab);
                    if (browser.contentDocument.location.href != url)
                        browser.contentDocument.location.href = url;
                    return browser;
                }
            }
        }
        return null;
    },
    
    browseNewTab: function(url) {
        log("Browse.browseNewTab(" + url + ")\n");
        
        var window = this.windowMediator().getMostRecentWindow("navigator:browser");
        if (window == null) return null;
        
        var tab = window.getBrowser().addTab(url);
        tab.id = MARKER;
        tab.addEventListener("DOMAttrModified", this.tabIconHandler, false);
        tab.setAttribute("image", this.tabIconHandler.icon);
        var browser = window.getBrowser().getBrowserForTab(tab);
        
        browser.parentNode.insertBefore(this.bar(window,
                                                 "Firedoclipse documentation",
                                                 "Documentation will be loaded in this browser tab"), browser);
        
        if (this.showUpdateMessage) {
            browser.parentNode.insertBefore(this.bar(window,
                                                     "An update is available for the Eclipse plug-in: go to Help > Software Updates > Installed Software > Update",
                                                     "You are running Firedoclipse " + this.firefoxVersion() + " in Firefox, and " + this.eclipseVersion + " in Eclipse"), browser);
        }
        
        return browser;
    },
    
    bar: function(window, text, tooltip) {
        function elt(tag) { return window.document.createElementNS("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul", tag); }
        function nest(parent, tag) { var e = elt(tag); parent.appendChild(e); return e; }
        
        var bar = elt("hbox");
        bar.className = "chromeclass-status";
        bar.setAttribute("style", "background:#6765b6");
                
        var panel = nest(bar, "statusbarpanel");
        panel.flex = 1;
        
        var label = nest(panel, "label");
        label.className = "statusbarpanel-text header";
        label.setAttribute("style", "color:white; font-size:xx-small; margin-top:1px; margin-bottom:2px;");
        label.flex = 1;
        label.textContent = text;
        label.setAttribute("tooltiptext", tooltip);
        
        return bar;
    },
    
    tabIconHandler: {
        icon: "chrome://firedoclipse/content/eclipse.gif",
        handleEvent: function(event) {
            if (event.attrName == "src" && event.newValue != this.icon) {
                event.target.setAttribute("image", this.icon);
            }
        }
    }
}

function NSGetModule(compMgr, fileSpec) { return XPCOMUtils.generateModule([ Browse ]) };
