/*
 * Firedoclipse
 * 
 * Copyright (c) 2007-2008 Massachusetts Institute of Technology
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package jsjr;

import static jsjr.test.XpcShell.js;
import static jsjr.test.XpcShell.xpcshell;
import static org.junit.Assert.assertEquals;

import java.io.File;
import java.io.IOException;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

public class XpConnectorsTest {
    
    static final File JSJR = new File("../jsjr/test/jsjr");
    static final int PORT = 3998;
    
    Connector connector;
    
    @Before public void setUp() throws IOException { }
    
    @After public void tearDown() throws IOException {
        if (connector != null)
            connector.close();
        connector = null;
        Thread.yield();
    }
    
    @Test(timeout=3000) public void testJsCallJava() throws Exception {
        connector = new ConnectorImpl(PORT);
        connector.accept();
        
        List<String> greetings = new ArrayList<String>();
        
        connector.export("greetings", greetings);
        
        xpcshell(
                js(new File(JSJR, "test/TestUtil.js")),
                js(new File(JSJR, "XpConnectorsTestJsCallJava.js")));
        
        assertEquals(
                Arrays.asList(new String[] { "Hello from JavaScript!" }),
                greetings);
    }
    
    @Test(timeout=3000) public void testJavaCallbackJs() throws Exception {
        connector = new ConnectorImpl(PORT);
        connector.accept();
        
        connector.export("bouncer", new TestBouncer());
        
        List<String> result = xpcshell(
                js(new File(JSJR, "test/TestUtil.js")),
                js(new File(JSJR, "XpConnectorsTestJavaCallbackJs.js")));
        
        assertEquals(
                Arrays.asList(new String[] { "The message is: 42" }),
                result);
    }
    
    @Test(timeout=3000) public void testJsBeforeJava() throws Exception {
        final List<String> result = new ArrayList<String>();
        
        Thread js = new Thread(new Runnable() {
            public void run() {
                try {
                    result.addAll(xpcshell(
                            js(new File(JSJR, "test/TestUtil.js")),
                            js(new File(JSJR, "XpConnectorsTestJsBeforeJava.js"))));
                } catch (IOException ioe) {
                    throw new RuntimeException(ioe);
                }
            }
        });
        js.start();
        
        Thread.sleep(800);
        
        connector = new ConnectorImpl(PORT);
        connector.accept();
        
        connector.export("bouncer", new TestBouncer());
        
        js.join();
        
        assertEquals(
                Arrays.asList(new String[] { "The message is: attack at dawn" }),
                result);
    }
}

class TestBouncer {
    /*
     * Bounce message to a recipient.
     */
    public void bounce(TestBouncee bouncee, String message) throws RemoteException {
        bouncee.receive(message);
    }
}

interface TestBouncee {
    public void receive(String message) throws RemoteException;
}

class TestBounceeProxy implements TestBouncee {
    
    private final Connection conn;
    private final String remoteId;
    
    public TestBounceeProxy(Connection conn, String remoteId) {
        this.conn = conn;
        this.remoteId = remoteId;
    }

    public void receive(String message) throws RemoteException {
        conn.call(remoteId, "receive", ConnectorImpl.marshalable(message));
    }
}
