/*
 * Firedoclipse
 * 
 * Copyright (c) 2007-2008 Massachusetts Institute of Technology
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

const Cc = Components.classes;
const Ci = Components.interfaces;

function utilities() {
  
  var waitForAsync;
  if (Cc["@mozilla.org/event-queue-service;1"]) {
    // MOZ Firefox 2
    eventQueues = Cc["@mozilla.org/event-queue-service;1"].getService(Ci.nsIEventQueueService);
    eventQueue = eventQueues.getSpecialEventQueue(eventQueues.CURRENT_THREAD_EVENT_QUEUE);
    
    waitForAsync = function(ms) {
      Cc['@mozilla.org/thread;1'].getService(Ci.nsIThread).currentThread.sleep(ms);
      eventQueue.processPendingEvents();
    };
    
  } else {
    // MOZ Firefox 3
    thread = Cc["@mozilla.org/thread-manager;1"].getService(Ci.nsIThreadManager).currentThread;
    
    waitForAsync = function(ms) {
      var sleeping = true;
      timer = Cc["@mozilla.org/timer;1"].createInstance(Ci.nsITimer);
      timer.init({ observe: function(s,t,d) { sleeping = false; } }, ms, Ci.nsITimer.TYPE_ONE_SHOT);
      while (sleeping) thread.processNextEvent(true);
    };
  }
  
  function getFile(path) {
    var comps = path.split("/");
    var file = Cc["@mozilla.org/file/directory_service;1"]
                   .getService(Ci.nsIProperties)
                       .get("CurWorkD", Ci.nsILocalFile);
    
    for (var ii = 0, size = comps.length; ii < size; ii++) {
      if (comps[ii].length > 0)
        file.append(comps[ii]);
    }
    
    if ( ! file.exists())
      throw file.path + " does not exist"
    
    return file;
  }

  return {
    register: function(path) {
      var file = getFile(path);
      Components.manager instanceof Ci.nsIComponentRegistrar;
      Components.manager.autoRegister(file);
    },
    
    waitForAsync: waitForAsync
  }
}

const util = utilities();
