/*
 * Firedoclipse
 * 
 * Copyright (c) 2007-2008 Massachusetts Institute of Technology
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package jsjr.test;

import java.io.*;
import java.util.*;

public class XpcShell {
    
    static final Properties PROPERTIES = new Properties();
    static {
        try {
            PROPERTIES.load(new FileInputStream("../jsjr/xpc.properties"));
        } catch (IOException ioe) { }
    }
    static final String XPCSHELL = PROPERTIES.getProperty("xpcshell");
    
    public static JavaScript js(String... javascripts) {
        StringBuilder sb = new StringBuilder();
        for (String js : javascripts) {
            sb.append(js);
            sb.append('\n');
        }
        return js(new StringReader(sb.toString()));
    }
    
    public static JavaScript js(File js) throws IOException {
        return js(new FileReader(js));
    }
    
    public static JavaScript js(final Reader js) {
        return new JavaScript() {
            public Reader reader() {
                return js;
            }
        };
    }
    
    public static List<String> xpcshell(JavaScript... javascripts) throws IOException {
        final Process xpcshell = Runtime.getRuntime().exec(XPCSHELL);
        Runtime.getRuntime().addShutdownHook(new Thread(new Runnable() {
            public void run() {
                xpcshell.destroy();
            }
        }));
        Writer stdin = new OutputStreamWriter(xpcshell.getOutputStream());
        
        char[] buff = new char[1024];
        int read;
        for (JavaScript js : javascripts) {
            Reader reader = js.reader();
            while ((read = reader.read(buff)) > 0) {
                stdin.write(buff, 0, read);
            }
        }
        stdin.close();
        
        BufferedReader stderr = new BufferedReader(new InputStreamReader(xpcshell.getErrorStream()));
        String line = stderr.readLine();
        if (line != null) throw new JavaScriptException(line);
        
        BufferedReader stdout = new BufferedReader(new InputStreamReader(xpcshell.getInputStream()));
        List<String> out = new ArrayList<String>();
        while ((line = stdout.readLine()) != null) {
            out.add(line);
        }
        return out;
    }
    
    public interface JavaScript {
        
        public Reader reader();
        
    }
}

class JavaScriptException extends RuntimeException {
    
    private static final long serialVersionUID = 1L;
    
    JavaScriptException(String message) {
        super(message);
    }
}
