/*
 * Firedoclipse
 * 
 * Copyright (c) 2007-2008 Massachusetts Institute of Technology
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

const Cc = Components.classes;
const Ci = Components.interfaces;

const EXTENSION_NAME = "jsjr";
const EXTENSION_HUMAN_NAME = "JSJR";

const THIS_DESCRIPTION = "logging";
const THIS_CID = Components.ID("{4cf7b9b0-d01b-11dc-95ff-0800200c9a66}");
const THIS_CONTRACTID = "@csail.mit.edu/jsjr/log;1";

const DirServiceId = "@mozilla.org/file/directory_service;1"
const FileOutStreamId = "@mozilla.org/network/file-output-stream;1"

/*
 * Logging.
 */

function Log() {
    this.wrappedJSObject = this;
}

Log.prototype = {
    init: function() {
        var log = Cc[DirServiceId]
                      .getService(Ci.nsIProperties)
                          .get("ProfD", Ci.nsIFile);
        log.append(EXTENSION_NAME + ".log");
        this.stream = Cc[FileOutStreamId].createInstance(Ci.nsIFileOutputStream);
        this.stream.init(log, 0x02 | 0x08 | 0x10 | 0x40, 0666, 0); // write, create, append, XXX sync
    },
    
    ok: function(message) {
        this.log(message + "\n");
    },
    
    info: function(message) {
        this.log("INFO " + message + "\n");
    },
    
    warn: function(message) {
        this.log("WARN " + message + "\n");
    },
    
    error: function(message) {
        this.log("ERROR " + message + "\n");
    },
    
    log: function(message) {
        message = new Date().toLocaleString() + ", " + message;
        this.stream.write(message, message.length);
    },
    
    QueryInterface: function(iid) {
        if ( ! iid.equals(Ci.nsISupports))
            throw Components.results.NS_ERROR_NO_INTERFACE;
        return this;
    }
}

/*
 * Singleton factory for QIable component, with initialization.
 */
function SingletonFactory() {
    this.instance = null;
}

SingletonFactory.prototype.createInstance = function(outer, iid) {
    if (outer != null) throw Components.results.NS_ERROR_NO_AGGREGATION;
    
    if (this.instance == null) {
        this.instance = new Log();
        this.instance.init();
    }
    
    return this.instance.QueryInterface(iid);
}

const THE_FACTORY = new SingletonFactory();

/*
 * Module automagically loaded by Firefox.
 */

var Module = new Object();

Module.registerSelf = function(compMgr, fileSpec, location, type) {
    compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);

    compMgr.registerFactoryLocation(THIS_CID,
                                    EXTENSION_HUMAN_NAME + " " + THIS_DESCRIPTION,
                                    THIS_CONTRACTID,
                                    fileSpec, location, type);
}

Module.unregisterSelf = function(compMgr, fileSpec, location) {
    compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);

    compMgr.unregisterFactoryLocation(THIS_CID, fileSpec);
}

Module.getClassObject = function(compMgr, cid, iid) {
    if ( ! iid.equals(Components.interfaces.nsIFactory))
        throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

    if (cid.equals(THIS_CID))
        return THE_FACTORY;

    throw Components.results.NS_ERROR_NO_INTERFACE;
}

Module.canUnload = function(compMgr) {
    return true;
}

function NSGetModule(compMgr, fileSpec) {
    return Module;
}
